using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Runtime.Serialization;
using System.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace GDBGTreeLines
{
   public class ExpandTreeLinesCellModel : GridDataBoundRowExpandCellModel
   {
      protected ExpandTreeLinesCellModel(SerializationInfo info, StreamingContext context)
         : base(info, context)
      {
      }

      public ExpandTreeLinesCellModel(GridModel grid)
         : base(grid)
      {
      }

		/// <override/>
      public override GridCellRendererBase CreateRenderer(GridControlBase control)
      {
         return new ExpandTreeLinesCellRenderer(control, this);
      }
	}

	public class ExpandTreeLinesCellRenderer : GridDataBoundRowExpandCellRenderer
	{
		GridDataBoundGrid dbGrid = null;
		private int offSet = 14;
		private int leftSide = 4;
		private int midPt = 5;
		private int indentLen = 8;
		private int centerLine = 4 + 5; //leftside + midPt;
		private Rectangle innerBounds;
		private int colWidth;
		public Color LineColor;

      /*
		ImageList imageList = new ImageList();
       */

		public ExpandTreeLinesCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{
			this.colWidth = leftSide + offSet + 2;

			this.dbGrid = grid as GridDataBoundGrid;
			this.dbGrid.Model.ResetVolatileData();
			this.dbGrid.CellDrawn += new GridDrawCellEventHandler(grid_CellDrawn);
			this.dbGrid.RowExpanding += new GridRowEventHandler(grid_RowExpanding);
			this.dbGrid.Model.QueryColWidth += new GridRowColSizeEventHandler(grid_QueryColWidth);
			this.dbGrid.Model.QueryCoveredRange += new GridQueryCoveredRangeEventHandler(grid_QueryCoveredRange);
			this.dbGrid.Paint += new PaintEventHandler(grid_Paint);
			this.dbGrid.ResizingColumns += new GridResizingColumnsEventHandler(grid_ResizingColumns);

			LineColor = Color.Plum;// Specify the Line Color..
			
         /*
			imageList.Images.Add(SystemIcons.Warning.ToBitmap());
			imageList.Images.Add(SystemIcons.Application.ToBitmap());
			imageList.Images.Add(SystemIcons.Asterisk.ToBitmap());
			imageList.ImageSize = new System.Drawing.Size(12, 12);
          */
		}


		protected override System.Drawing.Rectangle OnLayout(int rowIndex, int colIndex, Syncfusion.Windows.Forms.Grid.GridStyleInfo style, System.Drawing.Rectangle innerBounds, System.Drawing.Rectangle[] buttonsBounds)
		{
			Rectangle rect = base.OnLayout(rowIndex, colIndex, style, innerBounds, buttonsBounds);
			buttonsBounds[0].X = innerBounds.X + leftSide;
			this.innerBounds = innerBounds;

			if(rowIndex > this.dbGrid.Model.Rows.HeaderCount)
			{
				GridBoundRecordState state = this.dbGrid.Binder.GetRecordStateAtRowIndex(rowIndex);
				buttonsBounds[0].X = buttonsBounds[0].X + state.LevelIndex * offSet;
			}
			return rect;
		}

		private void DrawTreeLines(Graphics g, int rowIndex, Rectangle rect)
		{
			if(rowIndex > this.dbGrid.Model.Rows.HeaderCount)
			{
				using(Pen p = new Pen(LineColor))
				{
               // Draw tree lines dotted

					p.DashStyle = DashStyle.Dot;

					GridBoundRecordState state = this.dbGrid.Binder.GetRecordStateAtRowIndex(rowIndex);

               for(int level = 0; level <= state.LevelIndex; ++level)
					{
						int middle = this.innerBounds.Left + centerLine + level * this.offSet;

						if( level <	state.LevelIndex 
							&& (level < state.LevelIndex || state.Position < state.Table.Count - 1)
							)
						{
							GridBoundRecordState parentState = state.Parent;

                     while (parentState != null
                        && parentState.LevelIndex > level)
                     {
                        parentState = parentState.Parent;
                     }

						   if(parentState == null || parentState.Position < parentState.Table.Count - 1)
							{
								//draw long line on outer levels
								g.DrawLine(p, middle, this.innerBounds.Top, middle, this.innerBounds.Bottom); 
							}
						}
						else 
						{
							//draw the neck
							if(level > 0 || state.Position > 0)
								g.DrawLine(p, middle, this.innerBounds.Top, middle, rect.Top); 

							if(state.Position < state.Table.Count - 1) //draw the leg
								g.DrawLine(p, middle, rect.Bottom - 1, middle, this.innerBounds.Bottom); 

							//draw left arm
							g.DrawLine(p, rect.Right - 1, rect.Top + midPt, rect.Right + this.indentLen, rect.Top + midPt); 

							if(state.Expanded) //draw left arm bend
								g.DrawLine(p, rect.Right + this.indentLen, rect.Top + midPt, rect.Right + this.indentLen, this.innerBounds.Bottom); 

							GridStyleInfo style1 = this.dbGrid[rowIndex, 2];
							 
							Rectangle rect1 = this.GetCellClientRectangle(rowIndex, 2, style1, false);
							rect1.X = rect.Right + this.indentLen + 1;
							
                     /*
                     int index = state.LevelIndex;// choose an image somehow
							GridStaticCellRenderer.DrawImage(g, this.imageList, index, rect1, false);
							rect1.X += this.imageList.ImageSize.Width;
                      */

                     GridStaticCellRenderer.DrawText(g, style1.Text, style1.GdipFont, rect1, style1, style1.TextColor, false);
						}
					}
				}
			}
		}

		protected override void OnDrawCellButton(GridCellButton button, Graphics g, int rowIndex, int colIndex, bool bActive, Syncfusion.Windows.Forms.Grid.GridStyleInfo style)
		{
			//don't draw button on addnewrow
			if(this.dbGrid.Binder.AllowAddNew && rowIndex == this.dbGrid.Model.RowCount)
				return;

			base.OnDrawCellButton(button, g, rowIndex, colIndex, bActive, style);

			DrawTreeLines(g, rowIndex, button.Bounds);
		}

		private void grid_CellDrawn(object sender, GridDrawCellEventArgs e)
		{
			if(e.ColIndex == 1 && e.RowIndex > this.dbGrid.Model.Rows.HeaderCount
				&& (!this.dbGrid.Binder.AllowAddNew || e.RowIndex < this.dbGrid.Model.RowCount))
			{
		 	
				if(e.Style.Text.Length == 0 || Convert.ToInt32(e.Style.CellValue) == -1)
				{
					GridBoundRecordState state = this.dbGrid.Binder.GetRecordStateAtRowIndex(e.RowIndex);
				
					using(Pen p = new Pen(LineColor))
					{
						p.DashStyle = DashStyle.Dot;

						Rectangle rect = this.GetCellClientRectangle(e.RowIndex, e.ColIndex, e.Style, false);
						
                  for(int level = 0; level <= state.LevelIndex; ++level)
						{
							int middle = this.innerBounds.Left + centerLine + level * this.offSet;

							if( level <	state.LevelIndex)
							{
								GridBoundRecordState parentState = state.Parent;

                        while (parentState != null
                           && parentState.LevelIndex > level)
                        {
                           parentState = parentState.Parent;
                        }

                        if (parentState == null || parentState.Position < parentState.Table.Count - 1)
                        {
                           //draw long line on outer levels
                           e.Graphics.DrawLine(p, middle, rect.Top, middle, rect.Bottom);
                        }
							}
							else// if(state.Parent.Position < state.Parent.Table.Count - 1)
							{
								//draw the neck
								int middleVert = (rect.Top + rect.Bottom) / 2;
								e.Graphics.DrawLine(p, middle, rect.Top, middle, middleVert);

                        if (state.Position < state.Table.Count - 1) //draw the leg
                        {
                           e.Graphics.DrawLine(p, middle, middleVert, middle, rect.Bottom);
                        }

							   //draw left arm
								e.Graphics.DrawLine(p, middle, middleVert, middle + this.Model.ButtonBarSize.Width + this.indentLen, middleVert); 
							
								GridStyleInfo style1 = this.dbGrid[e.RowIndex, 2];
								 
								Rectangle rect1 = this.GetCellClientRectangle(e.RowIndex, 2, style1, false);
								rect1.X = middle + this.Model.ButtonBarSize.Width + this.indentLen + 1;

                        /*
								int index = 2;// choose an image somehow
								GridStaticCellRenderer.DrawImage(e.Graphics, this.imageList, index, rect1, false);
								rect1.X += this.imageList.ImageSize.Width;
                         */

								GridStaticCellRenderer.DrawText(e.Graphics, style1.Text, style1.GdipFont, rect1, style1, style1.TextColor, false);
							}
						}
					}
				}
			}
		}

		private void grid_RowExpanding(object sender, GridRowEventArgs e)
		{
			GridBoundRecordState state = this.dbGrid.Binder.GetRecordStateAtRowIndex(e.RowIndex);
			int width = leftSide + (state.LevelIndex + 2) * 14 + 2;

         if(this.colWidth < width)
			{
				this.colWidth = width;
				this.dbGrid.Refresh();
			}
		}

		private void grid_QueryColWidth(object sender, GridRowColSizeEventArgs e)
		{
			if(e.Index == 1)
			{
				e.Size = this.colWidth;
				e.Handled = true;
			}
		}

		private void grid_Paint(object sender, PaintEventArgs e)
		{
			this.dbGrid.Paint -= new PaintEventHandler(grid_Paint);
			this.dbGrid.Invalidate();
		}
		
		private void grid_QueryCoveredRange(object sender, GridQueryCoveredRangeEventArgs e)
		{
			if(e.RowIndex > this.dbGrid.Model.Rows.HeaderCount && (e.ColIndex == 1 || e.ColIndex == 2))
			{
				e.Range = GridRangeInfo.Cells(e.RowIndex, 1, e.RowIndex, 2);
				e.Handled = true;
			}
		}

		private void grid_ResizingColumns(object sender, GridResizingColumnsEventArgs e)
		{
         if (e.Columns.Left == 1)
         {
            e.Cancel = true;
         }
		}
	}
}
